use std::{env, ffi::OsString, path::PathBuf, process::exit, str::FromStr};

fn usage() -> ! {
    let name = env::args().next().unwrap();
    eprintln!(
        "usage: torbutt [-p PORT]

FLAGS (default -s if none set)
-s  deploy TCP server
-c  deploy TCP client

ARGS:
-p  port for TCP connection"
    );
    exit(1)
}

#[derive(Default)]
pub struct Args {
    pub port: Option<u16>,
}

impl Args {
    pub fn default() -> Self {
        Args { port: 8022 }
    }

    pub fn from_env() -> Self {
        let mut out = Self::default();
        let mut args = env::args_os().skip(1);
        while let Some(arg) = args.next() {
            let s = arg.to_string_lossy();
            let mut ch_iter = s.chars();
            if ch_iter.next() != Some('-') {
                out.positional.push(arg);
                continue;
            }
            ch_iter.for_each(|m| match m {
                // Edit these lines //
                'c' => out.config = parse_os_arg(args.next()),
                'd' => out.out_dir = parse_os_arg(args.next()),
                'g' => out.include_gemini = true,
                'h' => out.include_html = true,
                // Stop editing //
                _ => {
                    usage();
                }
            })
        }
        // other validation
        if out.positional.len() < 1 {
            usage()
        }
        out
    }
}

#[allow(dead_code)]
fn parse_arg<T: FromStr>(a: Option<OsString>) -> T {
    a.and_then(|a| a.into_string().ok())
        .and_then(|a| a.parse().ok())
        .unwrap_or_else(|| usage())
}
